@@ -39,6 +39,11 @@
 (require 'sendmail)
 (require 'shadow)
 
+(eval-when-compile
+  (defvar mh-before-send-letter-hook)
+  (defvar xemacs-betaname)
+  (defvar sxemacs-betaname))
+
 (defgroup xemacsbug nil
   "Sending XEmacs bug reports."
   :group 'maint
@@ -48,7 +53,7 @@
 ;; >> otherwise you can't use this file.  It will only work on the
 ;; >> internet with this address.
 
-(defcustom report-xemacs-bug-address "xemacs-beta@xemacs.org"
+(defcustom report-xemacs-bug-address "XEmacs Beta <xemacs-beta@xemacs.org>"
   "*Address of mailing list for XEmacs bugs."
   :group 'xemacsbug
   :type 'string)
@@ -63,13 +68,12 @@ strings. See `compose-mail'."
 	  (cons (string :tag "Header") 
 		(string :tag "Value"))))
 
-(defcustom report-xemacs-bug-beta-address "xemacs-beta@xemacs.org"
+(defcustom report-xemacs-bug-beta-address "XEmacs Beta <xemacs-beta@xemacs.org>"
   "*Address of mailing list for XEmacs beta bugs."
   :group 'xemacsbug
   :type 'string)
 
-(defcustom report-xemacs-bug-beta-extra-headers
-  '(("cc" . "ben@xemacs.org"))
+(defcustom report-xemacs-bug-beta-extra-headers nil
   "*An alist of mail-header value pairs for XEmacs beta bugs.
 
 It takes the format (HEADER . VALUE) where both HEADER and VALUE are
@@ -97,6 +101,57 @@ strings. See `compose-mail'."
   :group 'xemacsbug
   :type 'boolean)
 
+(defconst report-xemacs-bug-help
+"\nThis bug report will be sent to the XEmacs Development Team,
+not to your local site managers!!
+
+Please write in English, because the XEmacs maintainers do not have
+translators to read other languages for them.
+
+Please describe as succinctly as possible:
+\t- What happened.
+\t- What you thought should have happened.
+\t- Precisely what you were doing at the time.
+
+Also include a reliable recipe for triggering the bug, as well as
+any C and lisp back-traces that you may have.
+\(setq stack-trace-on-error t\), or \(setq debug-on-error t\) if you
+are familiar with the debugger, to get a lisp back-trace.
+To get a core file for the C back-trace on a GNU/Linux system do 
+'ulimit -c unlimited' in the shell prior to starting XEmacs.
+
+Type \\[report-xemacs-bug-info] to visit in Info the XEmacs Manual section
+about when and how to write a bug report,
+and what information to supply so that the bug can be fixed.
+Type SPC to scroll through this section and its subsections.
+
+You are very welcome to scan through the bug report and remove any
+potentially sensitive data.
+
+Turn off this help buffer permanently by adding:
+
+\t \(setq report-xemacs-bug-no-explanations t\)
+
+To your ~/.xemacs/init.el")
+
+(defun report-xemacs-bug-help ()
+  "Display the help buffer for `report-xemacs-bug'."
+  (with-electric-help
+   '(lambda () 
+      (define-key (current-local-map) "\C-c\C-i" 'report-xemacs-bug-info)
+      (princ (substitute-command-keys report-xemacs-bug-help)) nil) "*Bug Help*"))
+
+(defun report-xemacs-bug-packages-list ()
+  "Insert into the current buffer a list of installed packages."
+  (let ((pkgs packages-package-list))
+    (while pkgs
+      (insert
+       (format "(%s ver: %s upstream: %s)\n"
+	       (nth 0 (car pkgs))
+	       (nth 2 (car pkgs))
+	       (nth 4 (car pkgs))))
+      (setq pkgs (cdr pkgs)))))
+
 ;;;###autoload
 (defun report-xemacs-bug (topic &optional recent-keys)
   "Report a bug in XEmacs.
@@ -105,10 +160,11 @@ Prompts for bug subject.  Leaves you in a mail buffer."
   ;; the bug subject string is read.
   (interactive (reverse (list (recent-keys) (read-string "Bug Subject: "))))
   (let (user-point)
+    (setq topic (concat "[Bug: " emacs-program-version "] " topic))
     (if xemacs-betaname
-	(compose-mail report-xemacs-bug-beta-address
-		      topic
-		      report-xemacs-bug-beta-extra-headers)
+	  (compose-mail report-xemacs-bug-beta-address
+			topic
+			report-xemacs-bug-beta-extra-headers)
       (compose-mail report-xemacs-bug-address
 		    topic
 		    report-xemacs-bug-extra-headers))
@@ -117,22 +173,8 @@ Prompts for bug subject.  Leaves you in a mail buffer."
     (goto-char (point-min))
     (re-search-forward (concat "^" (regexp-quote mail-header-separator) "$"))
     (forward-line 1)
-    (unless report-xemacs-bug-no-explanations
-      ;; Insert warnings for novice users.
-      (insert "This bug report will be sent to the XEmacs Development Team,\n")
-      (insert-face " not to your local site managers!!" 'italic)
-      (insert "\nPlease write in ")
-      (insert-face "English" 'italic)
-      (insert ", because the XEmacs maintainers do not have
-translators to read other languages for them.\n\n"))
-    (insert "\n")
-    (insert "Please describe as succinctly as possible:
-\t- What happened.
-\t- What you thought should have happened.
-\t- Precisely what you were doing at the time.\n
-Please also include any C or lisp back-traces that you may have.
-================================================================
-Dear Bug Team!\n\n")
+    (insert "================================================================\n")
+    (insert "Dear Bug Team!\n\n")
     (setq user-point (point))
     (insert "\n\n================================================================\n
 System Info to help track down your bug:
@@ -155,12 +197,74 @@ System Info to help track down your bug:
 	  (narrow-to-region before-shadows (point))
 	  (fill-paragraph t)
 	  (insert "\n"))))
+
+    (insert "\n\Internationalization Settings:\n"
+	    "-------------------------\n")
+
+    (when (featurep 'mule)
+      (insert "\nEnvironment:\n\n")
+      (mapcar
+       (lambda (var)
+         (insert (format "  Value of %-12s: %s\n"
+                         var (getenv var))))
+       '("LC_ALL" "LC_COLLATE" "LC_CTYPE" "LC_MESSAGES"
+         "LC_MONETARY" "LC_NUMERIC" "LC_TIME" "LANG"))
+
+      (insert "\n")
+
+      (insert "Lisp locale settings:\n\n")
+
+      (dolist (sym '(current-language-environment
+                     default-buffer-file-coding-system
+                     default-process-coding-system
+                     (current-locale)
+                     keyboard-coding-system
+                     terminal-coding-system))
+        (insert (format "  %-34s=> %S\n" sym 
+                        (if (consp sym)
+                            (if (fboundp (car sym))
+                                (eval sym)
+                              [not available])
+                          (if (boundp sym)
+                              (symbol-value sym)
+                            [not available])))))
+
+      (insert (format "  %-34s=>\n" "(coding-priority-list)"))
+      (let ((before-coding-priority (point))
+            (fill-prefix "    "))
+        (insert (format "    %S" (coding-priority-list)))
+	(save-restriction
+	  (narrow-to-region before-coding-priority (point))
+	  (fill-paragraph nil)
+	  (insert "\n")))
+
+      (insert "\n")
+
+      (insert "Coding system aliases:\n\n")
+
+      (dolist (alias '(native file-name
+                       mswindows-multibyte-system-default))
+        (insert (if (not (coding-system-alias-p alias))
+                    (format "  '%-35s is not a coding system alias\n"
+                            alias)
+                  (format "  '%-35s is aliased to %S\n"
+                          alias
+                          (coding-system-aliasee alias))))))
+
     ;; Insert a list of installed packages.
     (insert "\n\nInstalled XEmacs Packages:\n"
 	    "-------------------------\n")
-    (cl-prettyprint
-     (symbol-value 'packages-package-list))
+    (report-xemacs-bug-packages-list)
     (insert "\n")
+    ;; Insert a list of installed modules.
+    (if (fboundp 'list-modules)
+	(progn
+	  (insert "\n\nInstalled Modules:\n"
+		  "-----------------\n")
+	    (let* ((mods (list-modules)))
+	      (while mods
+		(cl-prettyprint (cdr (car mods)))
+		(setq mods (cdr mods))))))
     ;; Insert a list of loaded features
     (let ((before-features (point)))
       (insert
@@ -200,53 +304,55 @@ System Info to help track down your bug:
 		     (set-keymap-parents map (current-local-map))
 		     map))
     (define-key (current-local-map) "\C-c\C-i" 'report-xemacs-bug-info)
-    (unless report-xemacs-bug-no-explanations
-      (with-output-to-temp-buffer "*Bug Help*"
-	(cond
-	 ((eq mail-user-agent 'sendmail-user-agent)
-	  (princ (substitute-command-keys
-		  "Type \\[mail-send-and-exit] to send the bug report.\n")))
-	 ((eq mail-user-agent 'message-user-agent)
-	  (princ (substitute-command-keys
-		  "Type \\[message-send-and-exit] to send the bug report.\n")))
-	 ((eq mail-user-agent 'mh-e-user-agent)
-	  (princ (substitute-command-keys
-		  "Type \\[mh-send-letter] to send the bug report.\n")))
-	 (t
-	  (princ (substitute-command-keys
-		  "Type \\[mail-send-and-exit] to send the bug report.\n"))))
-	(princ (substitute-command-keys
-		"Type \\[kill-buffer] RET to cancel (don't send it).\n"))
-	(terpri)
-	(princ (substitute-command-keys
-		"\
-Type \\[report-xemacs-bug-info] to visit in Info the XEmacs Manual section
-about when and how to write a bug report,
-and what information to supply so that the bug can be fixed.
-Type SPC to scroll through this section and its subsections."))
-	(princ "\n\n
-You are very welcome to scan through the bug report and remove any
-potentially sensitive data.")))
     ;; Make it less likely people will send empty messages.
-    (make-local-variable 'mail-send-hook)
-    (add-hook 'mail-send-hook 'report-xemacs-bug-hook)
+    (cond
+     ((eq mail-user-agent 'sendmail-user-agent)
+      (make-local-variable 'mail-send-hook)
+      (add-hook 'mail-send-hook 'report-xemacs-bug-hook))
+     ((eq mail-user-agent 'message-user-agent)
+      (make-local-variable 'message-send-hook)
+      (add-hook 'message-send-hook 'report-xemacs-bug-hook))
+     ((eq mail-user-agent 'mh-e-user-agent)
+      (make-local-variable 'mh-before-send-letter-hook)
+      (add-hook 'mh-before-send-letter-hook 'report-xemacs-bug-hook))
+     (t
+      (make-local-variable 'mail-send-hook)
+      (add-hook 'mail-send-hook 'report-xemacs-bug-hook)))
     (save-excursion
       (goto-char (point-max))
       (skip-chars-backward " \t\n")
       (make-local-variable 'report-xemacs-bug-orig-text)
       (setq report-xemacs-bug-orig-text (buffer-substring (point-min) (point))))
-    (goto-char user-point)))
+    (goto-char user-point))
+  (delete-other-windows)
+  (unless report-xemacs-bug-no-explanations
+    (report-xemacs-bug-help)
+    (cond
+     ((eq mail-user-agent 'sendmail-user-agent)
+      (message (substitute-command-keys
+		"Type \\[mail-send-and-exit] to send the bug report, \\[kill-buffer] to cancel.")))
+     ((eq mail-user-agent 'message-user-agent)
+      (message (substitute-command-keys
+		"Type \\[message-send-and-exit] to send the bug report, \\[kill-buffer] to cancel.")))
+     ((eq mail-user-agent 'mh-e-user-agent)
+      (message (substitute-command-keys
+		"Type \\[mh-send-letter] to send the bug report, \\[kill-buffer] to cancel.")))
+     (t
+      (message (substitute-command-keys
+		"Type \\[mail-send-and-exit] to send the bug report, \\[kill-buffer] to cancel."))))))
+
 
 ;; For backwards compatibility
 ;;;###autoload
 (defalias 'report-emacs-bug 'report-xemacs-bug)
 
 (defun report-xemacs-bug-info ()
-  "Go to the Info node on reporting Emacs bugs."
+  "Go to the Info node on reporting XEmacs bugs."
   (interactive)
   (Info-goto-node "(xemacs)Bugs"))
 
 (defun report-xemacs-bug-hook ()
+  "Hook run before sending a bug report."
   (save-excursion
     (goto-char (point-max))
     (skip-chars-backward " \t\n")
@@ -256,19 +362,6 @@ potentially sensitive data.")))
 		    report-xemacs-bug-orig-text))
 	(error "No text entered in bug report"))
 
-    ;; Check the buffer contents and reject non-English letters.
-    (save-excursion
-      (goto-char (point-min))
-      (skip-chars-forward "\0-\177")
-      (if (not (eobp))
-	  (if (or report-xemacs-bug-no-confirmation
-		  (y-or-n-p "Convert non-ASCII letters to hexadecimal? "))
-	      (while (progn (skip-chars-forward "\0-\177")
-			    (not (eobp)))
-		(let ((ch (following-char)))
-		  (delete-char 1)
-		  (insert (format "=%02x" ch)))))))
-
     ;; The last warning for novice users.
     (if (or report-xemacs-bug-no-confirmation
 	    (yes-or-no-p
@@ -276,20 +369,40 @@ potentially sensitive data.")))
 	;; Just send the current mail.
 	nil
       (goto-char (point-min))
-      (if (search-forward "To: ")
-	  (let ((pos (point)))
-	    (end-of-line)
-	    (delete-region pos (point))))
-      (kill-local-variable 'mail-send-hook)
-      (with-output-to-temp-buffer "*Bug Help*"
-	(princ (substitute-command-keys "\
+      (let* ((top (point)))
+	(re-search-forward (concat "^" (regexp-quote mail-header-separator) "$"))
+	(save-restriction
+	  (narrow-to-region top (point))
+	  (goto-char (point-min))
+	  (if (re-search-forward "^To: " (eobp) t)
+	      (let ((pos (point)))
+		(end-of-line)
+		(delete-region pos (point))))
+	  (goto-char (point-min))
+	  (if (re-search-forward "^Cc: " (eobp) t)
+	      (let ((pos (point)))
+		(end-of-line)
+		(delete-region pos (point))))))
+      (cond
+       ((eq mail-user-agent 'sendmail-user-agent)
+	(kill-local-variable 'mail-send-hook))
+       ((eq mail-user-agent 'message-user-agent)
+	(kill-local-variable 'message-send-hook))
+       ((eq mail-user-agent 'mh-e-user-agent)
+	(kill-local-variable 'mh-before-send-letter-hook))
+       (t
+	(kill-local-variable 'mail-send-hook)))
+      (unless report-xemacs-bug-no-explanations
+	(with-electric-help
+	 '(lambda ()
+	    (insert "\n
 You invoked the command M-x report-xemacs-bug,
 but you decided not to mail the bug report to the XEmacs maintainers.
 
 If you want to mail it to someone else instead,
 please insert the proper e-mail address after \"To: \",
-and send the mail again using \\[mail-send-and-exit].")))
-      (error "M-x report-xemacs-bug was cancelled, please read *Bug Help* buffer"))))
+and send the mail again.") nil) "*Bug Help*"))
+      (error "Sending Bug Report Cancelled"))))
 
 (provide 'xemacsbug)
 
